"use strict";(globalThis["webpackChunkuniqua"]=globalThis["webpackChunkuniqua"]||[]).push([[193],{2193:(e,t,n)=>{n.r(t),n.d(t,{default:()=>p});var l=n(3673);const o={class:"fullscreen bg-blue text-white text-center q-pa-md flex flex-center"},s=(0,l._)("div",{style:{"font-size":"30vh"}}," 404 ",-1),a=(0,l._)("div",{class:"text-h2",style:{opacity:".4"}}," Oops. Nothing here... ",-1);function c(e,t,n,c,r,u){const i=(0,l.up)("q-btn");return(0,l.wg)(),(0,l.iD)("div",o,[(0,l._)("div",null,[s,a,(0,l.Wm)(i,{class:"q-mt-xl",color:"white","text-color":"blue",unelevated:"",to:"/",label:"Go Home","no-caps":""})])])}const r=(0,l.aZ)({name:"Error404"});var u=n(4260),i=n(8240),h=n(7518),b=n.n(h);const d=(0,u.Z)(r,[["render",c]]),p=d;b()(r,"components",{QBtn:i.Z})}}]);